%% Parameters
syms m g IG l w real % rigid body parameters
syms ks l0      real % spring parameters

% Coordinates
syms x xd xdd real % x, xdot, and xddot
syms y yd ydd real % y, ydot, and yddot
syms t td tdd real % theta, thetadot, and thetaddot


%% Reference frames
% Inertial frame
i = [1; 0; 0];
j = [0; 1; 0];
k = [0; 0; 1];

% Body frame
b1 =  cos(t)*i + sin(t)*j;
b2 = -sin(t)*i + cos(t)*j;
b3 =  k;


%% Kinematics
r_OG = x*i + y*j;       % Position from O to G
v_OG = xd*i + yd*j;     % Velocity from O to G
a_OG = xdd*i + ydd*j;   % Acceleration from O to G

r_GA = l/2*b1 + w/2*b2; % Position from G to A
r_OA = r_OG + r_GA;     % Position from O to A


%% Forces
Fg = -m*g*j; % Gravity force
Fs = ks*(norm(r_OA) - l0)*(-r_OA/norm(r_OA)); % Spring force


%% LMB
LMB = Fg + Fs == m*a_OG;
eq1 = LMB(1); % in x direction
eq2 = LMB(2); % in y direction

%% AMB about A
AMB_A = cross(-r_GA,Fg) == IG*tdd*k + cross(-r_GA,m*a_OG);
eq3 = AMB_A(3); % only take the 3rd equation


%% EOM
eom = simplify( [eq1; eq2; eq3] );
[xdd, ydd, tdd] = solve(eom, [xdd, ydd, tdd]);

matlabFunction(xdd,'File','xdd')
matlabFunction(ydd,'File','ydd')
matlabFunction(tdd,'File','tdd')

