%% Parameters
p.m = 1;
p.g = 10;
p.IG = 1;
p.l = 2;
p.w = 1;
p.ks = 1;
p.l0 = 5;


%% Initialization
t0 = 0;
tf = 10;
tspan = linspace(t0,tf,1e3);


%% Initial conditions
X0 = 0;
Y0 = -5;
TH0 = atan2(p.l/2, p.w/2);
Xd0 = -1;
Yd0 = 0;
THd0 = 0;
Z0 = [X0; Y0; TH0; Xd0; Yd0; THd0];


%% ODE initialization
f   = @(t,z)  mybungy(t,z,p); % rhs function
tol	= 1e-7;                 % accuracy
opt = odeset('RelTol', tol, 'AbsTol', tol);


%% Solution
[tarray, zarray]  = ode45(f, tspan, Z0, opt);


%% Unpack variables
xG = zarray(:,1);       xGdot = zarray(:,4);
yG = zarray(:,2);       yGdot = zarray(:,5);
th = zarray(:,3);       thdot = zarray(:,6);


%% Plotting
% Trajectory of COM
figure
hold on
grid on
axis equal
plot(xG,yG,'k-')
plot(xG(1),yG(1),'ro')
plot(xG(end),yG(end),'rx')
xlabel('X-coordinate of COM')
ylabel('Y-coordinate of COM')
legend({'Path','Start', 'End'})
hold off

% Time histories
figure
subplot(2,1,1)
hold on
grid on
plot(tarray,xG,'b-')
plot(tarray,yG,'r-')
plot(tarray,th,'k-')
legend({'xG','yG','th'})
hold off
subplot(2,1,2)
hold on
grid on
plot(tarray,xGdot,'b-')
plot(tarray,yGdot,'r-')
plot(tarray,thdot,'k-')
legend({'xGdot','yGdot','thdot'})
hold off


%% Animation

S = ode45(f, tspan, Z0, opt);

bungee = figure;
set( bungee, 'DoubleBuffer', 'on' )
hold on
grid on
axis equal
xlabel('X')
ylabel('Y')
title('Animation of bungy jumper trajectory')

X = S.y(1,:);
Y = S.y(2,:);
dist = p.l + p.w;

ax_lim = [min(X)-dist, max(X)+dist, min(Y)-dist, max(Y)+dist];
axis(ax_lim)

x(1) = 0;   y(1) = 0;
x(2) = 0;   y(2) = 0;
x(3) = 0;   y(3) = 0;
x(4) = 0;   y(4) = 0;
x(5) = 0;   y(5) = 0;

person = plot(x,y,'r');
spring = plot([0,x(1)],[0,x(2)],'k');

t_temp = 0;
tic;

while (t_temp < S.x(end))
    
    % Evaluate at real time
    soln = deval(S,t_temp);
    X   = soln(1,:);
    Y   = soln(2,:);
    TH  = soln(3,:);
    
    % Corners of square
    x1 = X + (p.l/2)*cos(TH) - (p.w/2)*sin(TH);
    x2 = X - (p.l/2)*cos(TH) - (p.w/2)*sin(TH);
    x3 = X - (p.l/2)*cos(TH) + (p.w/2)*sin(TH);
    x4 = X + (p.l/2)*cos(TH) + (p.w/2)*sin(TH);
    
    y1 = Y + (p.l/2)*sin(TH) + (p.w/2)*cos(TH);
    y2 = Y - (p.l/2)*sin(TH) + (p.w/2)*cos(TH);
    y3 = Y - (p.l/2)*sin(TH) - (p.w/2)*cos(TH);
    y4 = Y + (p.l/2)*sin(TH) - (p.w/2)*cos(TH);
    
    % Transform person and spring
    set(person,'Xdata',[x1 x2 x3 x4 x1])
    set(person,'Ydata',[y1 y2 y3 y4 y1])
    set(spring,'Xdata',[0, x1])
    set(spring,'Ydata',[0, y1])
    
    drawnow
    
    t_temp  = toc;
    fprintf('t = %d\n',t_temp)
    
end

hold off

%% RHS function
function dZ = mybungy(~,Z,p)

% Unpack parameters
m = p.m;
g = p.g;
IG = p.IG ;
l = p.l;
w = p.w;
ks = p.ks;
l0 = p.l0;

% Unpack state
x       = Z(1);
y       = Z(2);
th      = Z(3);

Xdd     = xdd(ks,l,l0,m,th,w,x,y);
Ydd     = ydd(g,ks,l,l0,m,th,w,x,y);
THdd    = tdd(IG,ks,l,l0,th,w,x,y);

dZ      = [Z(4:6); Xdd; Ydd; THdd];

end